clear all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                DIVIDE NETWORK INTO CLUSTERS USING ADMSC
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  STEP 0. Set up the beta factor and number of clusters.
%  STEP 1. Prepare the network
%             You prepare the edge list(input) file and then use the function
%                  edges2net
%             to get the adjacency matrix.
%  STEP 2. Preprocess the network
%          a) You find components by using the function
%                  components
%             and then treat each component as a network individually.
%          b) If your network has multiple components, 
%             you can find the core network by using the function
%                  corecom
%             to pick out the network.
%  STEP 3. Decide the number of clusters
%          a) Calculate the eigenvalues and eigenvectors by using
%                    eigmap
%          *) Observe the eigenvalue curve, find the greatest gap. The
%             position of the gap is the number of clusters.
%             If there is no obvious gap, the network does not have a 
%             clear clusterable structure. However, you can still force the
%             algorithm to form some "clusters" as you want. In this case,
%             simply choose a number you like.
%  STEP 4. Find the clusters by using the function
%                    dcluster
%          which divide the network into clusters.
%  STEP 5. Check and evaluate the results.
%          a) Visualize the connection density by
%                    adjplot
%             Good clustering results in dark diagonal blocks.
%          b) Evaluate results by calculating numeric indices, using
%                    assess_cluster
%             Roughly speaking, larger numbers imply better results.
%             BE CAREFUL: 
%                    The indices are not satisfactory. 
%                    (Well, of course, better than none.)
%  STEP 6. Calculate coefficiant of variation of clustering result
%========================== STEP 0 ===============================
nc=33; % Number of clusters
B=1.4; % Beta factor

%========================== STEP 1 ===============================
% Here is a model network generator, which produces your network.
[nodea nodeb] = textread('Scere.txt','%s%s');  % input your network
[A nodes] = edges2net(nodea,nodeb,0);

%========================== STEP 2 ===============================
ix = components(A);
% ix = component index,
% if your network has multiple components, the core network is created.
if max(ix)>1
    fprintf('There are %4d components. The largest component is chosen.\n',max(ix));
    [A nodes] = corecom(ix,A,nodes);
end
nnode=length(A);
nedge=nnz(A)/2;
fprintf('Number of Nodes: %d . Number of Edges: %d .\n',nnode,nedge);

%========================== STEP 3 ===============================
tic
[V lam]=eigmap(A,nc,B);

% if you want to see the eigengap, it is possible to see by excluding
% the following comments.
%{
figure; 
plot(2:length(lam),lam(2:end),'-bo');
xlabel('Rank number');
ylabel('Eigenvalue');
grid on
title('Smallest eigenvalues of the network');
%}

%========================== STEP 4 ===============================
cidx = dcluster(V,nc,'cosine');
%cidx = cluster index,  1 <= cidx <= nc
 %cidx(i) = number of the cluster that node i belongs to
toc
%========================== STEP 5 ===============================
figure;
fh=subplot(1,1,1); adjplot(A,cidx,fh,2)

modu = assess_cluster(A,cidx);
fprintf('modularity = %6.3f\n',modu)

%========================== STEP 6 ===============================
cv=ClusterSize(cidx);

%========================== STEP 7 ===============================
outputdata(nodes,cidx,modu,cv,nnode,nedge); % output data

%========================== STEP 8 (options) =====================
% If you want to calculate the degree distribution or the coefficient of
% cluster or to convert into the Pajek format, it is possible by excluding
% the following comments respectively.

% Digree distribution
%degree(A);
% Coefficiant of cluster
%CC(A);
% Pajek network format
%pajek_net(A,nodes);
%pajek_clu(cidx);